# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.4.8
    Generated by: https://openapi-generator.tech
"""

import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401
from datetime import date, datetime  # noqa: F401

import frozendict  # noqa: F401
import typing_extensions  # noqa: F401

from gentrace import schemas  # noqa: F401

class FeedbackRequest(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "score",
            "pipelineRunId",
            "recordedTime",
        }

        class properties:
            pipelineRunId = schemas.UUIDSchema

            class score(schemas.Float32Schema):
                pass
            recordedTime = schemas.DateTimeSchema

            class details(
                schemas.StrBase, schemas.NoneBase, schemas.Schema, schemas.NoneStrMixin
            ):
                def __new__(
                    cls,
                    *_args: typing.Union[
                        None,
                        str,
                    ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> "details":
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            __annotations__ = {
                "pipelineRunId": pipelineRunId,
                "score": score,
                "recordedTime": recordedTime,
                "details": details,
            }
        additional_properties = schemas.NotAnyTypeSchema
    score: MetaOapg.properties.score
    pipelineRunId: MetaOapg.properties.pipelineRunId
    recordedTime: MetaOapg.properties.recordedTime

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["score"]
    ) -> MetaOapg.properties.score: ...
    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["pipelineRunId"]
    ) -> MetaOapg.properties.pipelineRunId: ...
    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["recordedTime"]
    ) -> MetaOapg.properties.recordedTime: ...
    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["details"]
    ) -> MetaOapg.properties.details: ...
    def __getitem__(
        self,
        name: typing.Union[
            typing_extensions.Literal["score"],
            typing_extensions.Literal["pipelineRunId"],
            typing_extensions.Literal["recordedTime"],
            typing_extensions.Literal["details"],
        ],
    ):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["score"]
    ) -> MetaOapg.properties.score: ...
    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["pipelineRunId"]
    ) -> MetaOapg.properties.pipelineRunId: ...
    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["recordedTime"]
    ) -> MetaOapg.properties.recordedTime: ...
    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["details"]
    ) -> typing.Union[MetaOapg.properties.details, schemas.Unset]: ...
    def get_item_oapg(
        self,
        name: typing.Union[
            typing_extensions.Literal["score"],
            typing_extensions.Literal["pipelineRunId"],
            typing_extensions.Literal["recordedTime"],
            typing_extensions.Literal["details"],
        ],
    ):
        return super().get_item_oapg(name)
    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        score: typing.Union[
            MetaOapg.properties.score,
            decimal.Decimal,
            int,
            float,
        ],
        pipelineRunId: typing.Union[
            MetaOapg.properties.pipelineRunId,
            str,
            uuid.UUID,
        ],
        recordedTime: typing.Union[
            MetaOapg.properties.recordedTime,
            str,
            datetime,
        ],
        details: typing.Union[
            MetaOapg.properties.details, None, str, schemas.Unset
        ] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
    ) -> "FeedbackRequest":
        return super().__new__(
            cls,
            *_args,
            score=score,
            pipelineRunId=pipelineRunId,
            recordedTime=recordedTime,
            details=details,
            _configuration=_configuration,
        )



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints then import them from
# tags, paths, or path_to_api, or tag_to_api



import typing_extensions

from gentrace.apis.tags import TagValues
from gentrace.apis.tags.feedback_api import FeedbackApi
from gentrace.apis.tags.ingestion_api import IngestionApi

TagToApi = typing_extensions.TypedDict(
    "TagToApi",
    {
        TagValues.INGESTION: IngestionApi,
        TagValues.FEEDBACK: FeedbackApi,
    },
)

tag_to_api = TagToApi(
    {
        TagValues.INGESTION: IngestionApi,
        TagValues.FEEDBACK: FeedbackApi,
    }
)



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.4.8
    Generated by: https://openapi-generator.tech
"""

import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401
from datetime import date, datetime  # noqa: F401

import frozendict  # noqa: F401
import typing_extensions  # noqa: F401

from gentrace import schemas  # noqa: F401


class PipelineRunRequest(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "stepRuns",
            "name",
            "id",
        }

        class properties:
            id = schemas.UUIDSchema
            name = schemas.StrSchema

            class stepRuns(schemas.ListSchema):
                class MetaOapg:
                    class items(schemas.DictSchema):
                        class MetaOapg:
                            class properties:
                                class provider(schemas.DictSchema):
                                    class MetaOapg:
                                        class properties:
                                            name = schemas.StrSchema
                                            invocation = schemas.StrSchema

                                            class modelParams(schemas.DictSchema):
                                                class MetaOapg:
                                                    additional_properties = (
                                                        schemas.AnyTypeSchema
                                                    )

                                                def __getitem__(
                                                    self, name: typing.Union[str,]
                                                ) -> MetaOapg.additional_properties:
                                                    # dict_instance[name] accessor
                                                    return super().__getitem__(name)

                                                def get_item_oapg(
                                                    self, name: typing.Union[str,]
                                                ) -> MetaOapg.additional_properties:
                                                    return super().get_item_oapg(name)

                                                def __new__(
                                                    cls,
                                                    *_args: typing.Union[
                                                        dict,
                                                        frozendict.frozendict,
                                                    ],
                                                    _configuration: typing.Optional[
                                                        schemas.Configuration
                                                    ] = None,
                                                    **kwargs: typing.Union[
                                                        MetaOapg.additional_properties,
                                                        dict,
                                                        frozendict.frozendict,
                                                        str,
                                                        date,
                                                        datetime,
                                                        uuid.UUID,
                                                        int,
                                                        float,
                                                        decimal.Decimal,
                                                        bool,
                                                        None,
                                                        list,
                                                        tuple,
                                                        bytes,
                                                        io.FileIO,
                                                        io.BufferedReader,
                                                    ],
                                                ) -> "modelParams":
                                                    return super().__new__(
                                                        cls,
                                                        *_args,
                                                        _configuration=_configuration,
                                                        **kwargs,
                                                    )

                                            class inputs(schemas.DictSchema):
                                                class MetaOapg:
                                                    additional_properties = (
                                                        schemas.AnyTypeSchema
                                                    )

                                                def __getitem__(
                                                    self, name: typing.Union[str,]
                                                ) -> MetaOapg.additional_properties:
                                                    # dict_instance[name] accessor
                                                    return super().__getitem__(name)

                                                def get_item_oapg(
                                                    self, name: typing.Union[str,]
                                                ) -> MetaOapg.additional_properties:
                                                    return super().get_item_oapg(name)

                                                def __new__(
                                                    cls,
                                                    *_args: typing.Union[
                                                        dict,
                                                        frozendict.frozendict,
                                                    ],
                                                    _configuration: typing.Optional[
                                                        schemas.Configuration
                                                    ] = None,
                                                    **kwargs: typing.Union[
                                                        MetaOapg.additional_properties,
                                                        dict,
                                                        frozendict.frozendict,
                                                        str,
                                                        date,
                                                        datetime,
                                                        uuid.UUID,
                                                        int,
                                                        float,
                                                        decimal.Decimal,
                                                        bool,
                                                        None,
                                                        list,
                                                        tuple,
                                                        bytes,
                                                        io.FileIO,
                                                        io.BufferedReader,
                                                    ],
                                                ) -> "inputs":
                                                    return super().__new__(
                                                        cls,
                                                        *_args,
                                                        _configuration=_configuration,
                                                        **kwargs,
                                                    )

                                            class outputs(schemas.DictSchema):
                                                class MetaOapg:
                                                    additional_properties = (
                                                        schemas.AnyTypeSchema
                                                    )

                                                def __getitem__(
                                                    self, name: typing.Union[str,]
                                                ) -> MetaOapg.additional_properties:
                                                    # dict_instance[name] accessor
                                                    return super().__getitem__(name)

                                                def get_item_oapg(
                                                    self, name: typing.Union[str,]
                                                ) -> MetaOapg.additional_properties:
                                                    return super().get_item_oapg(name)

                                                def __new__(
                                                    cls,
                                                    *_args: typing.Union[
                                                        dict,
                                                        frozendict.frozendict,
                                                    ],
                                                    _configuration: typing.Optional[
                                                        schemas.Configuration
                                                    ] = None,
                                                    **kwargs: typing.Union[
                                                        MetaOapg.additional_properties,
                                                        dict,
                                                        frozendict.frozendict,
                                                        str,
                                                        date,
                                                        datetime,
                                                        uuid.UUID,
                                                        int,
                                                        float,
                                                        decimal.Decimal,
                                                        bool,
                                                        None,
                                                        list,
                                                        tuple,
                                                        bytes,
                                                        io.FileIO,
                                                        io.BufferedReader,
                                                    ],
                                                ) -> "outputs":
                                                    return super().__new__(
                                                        cls,
                                                        *_args,
                                                        _configuration=_configuration,
                                                        **kwargs,
                                                    )

                                            __annotations__ = {
                                                "name": name,
                                                "invocation": invocation,
                                                "modelParams": modelParams,
                                                "inputs": inputs,
                                                "outputs": outputs,
                                            }

                                    @typing.overload
                                    def __getitem__(
                                        self, name: typing_extensions.Literal["name"]
                                    ) -> MetaOapg.properties.name:
                                        ...

                                    @typing.overload
                                    def __getitem__(
                                        self,
                                        name: typing_extensions.Literal["invocation"],
                                    ) -> MetaOapg.properties.invocation:
                                        ...

                                    @typing.overload
                                    def __getitem__(
                                        self,
                                        name: typing_extensions.Literal["modelParams"],
                                    ) -> MetaOapg.properties.modelParams:
                                        ...

                                    @typing.overload
                                    def __getitem__(
                                        self, name: typing_extensions.Literal["inputs"]
                                    ) -> MetaOapg.properties.inputs:
                                        ...

                                    @typing.overload
                                    def __getitem__(
                                        self, name: typing_extensions.Literal["outputs"]
                                    ) -> MetaOapg.properties.outputs:
                                        ...

                                    @typing.overload
                                    def __getitem__(
                                        self, name: str
                                    ) -> schemas.UnsetAnyTypeSchema:
                                        ...

                                    def __getitem__(
                                        self,
                                        name: typing.Union[
                                            typing_extensions.Literal[
                                                "name",
                                                "invocation",
                                                "modelParams",
                                                "inputs",
                                                "outputs",
                                            ],
                                            str,
                                        ],
                                    ):
                                        # dict_instance[name] accessor
                                        return super().__getitem__(name)

                                    @typing.overload
                                    def get_item_oapg(
                                        self, name: typing_extensions.Literal["name"]
                                    ) -> typing.Union[
                                        MetaOapg.properties.name, schemas.Unset
                                    ]:
                                        ...

                                    @typing.overload
                                    def get_item_oapg(
                                        self,
                                        name: typing_extensions.Literal["invocation"],
                                    ) -> typing.Union[
                                        MetaOapg.properties.invocation, schemas.Unset
                                    ]:
                                        ...

                                    @typing.overload
                                    def get_item_oapg(
                                        self,
                                        name: typing_extensions.Literal["modelParams"],
                                    ) -> typing.Union[
                                        MetaOapg.properties.modelParams, schemas.Unset
                                    ]:
                                        ...

                                    @typing.overload
                                    def get_item_oapg(
                                        self, name: typing_extensions.Literal["inputs"]
                                    ) -> typing.Union[
                                        MetaOapg.properties.inputs, schemas.Unset
                                    ]:
                                        ...

                                    @typing.overload
                                    def get_item_oapg(
                                        self, name: typing_extensions.Literal["outputs"]
                                    ) -> typing.Union[
                                        MetaOapg.properties.outputs, schemas.Unset
                                    ]:
                                        ...

                                    @typing.overload
                                    def get_item_oapg(
                                        self, name: str
                                    ) -> typing.Union[
                                        schemas.UnsetAnyTypeSchema, schemas.Unset
                                    ]:
                                        ...

                                    def get_item_oapg(
                                        self,
                                        name: typing.Union[
                                            typing_extensions.Literal[
                                                "name",
                                                "invocation",
                                                "modelParams",
                                                "inputs",
                                                "outputs",
                                            ],
                                            str,
                                        ],
                                    ):
                                        return super().get_item_oapg(name)

                                    def __new__(
                                        cls,
                                        *_args: typing.Union[
                                            dict,
                                            frozendict.frozendict,
                                        ],
                                        name: typing.Union[
                                            MetaOapg.properties.name, str, schemas.Unset
                                        ] = schemas.unset,
                                        invocation: typing.Union[
                                            MetaOapg.properties.invocation,
                                            str,
                                            schemas.Unset,
                                        ] = schemas.unset,
                                        modelParams: typing.Union[
                                            MetaOapg.properties.modelParams,
                                            dict,
                                            frozendict.frozendict,
                                            schemas.Unset,
                                        ] = schemas.unset,
                                        inputs: typing.Union[
                                            MetaOapg.properties.inputs,
                                            dict,
                                            frozendict.frozendict,
                                            schemas.Unset,
                                        ] = schemas.unset,
                                        outputs: typing.Union[
                                            MetaOapg.properties.outputs,
                                            dict,
                                            frozendict.frozendict,
                                            schemas.Unset,
                                        ] = schemas.unset,
                                        _configuration: typing.Optional[
                                            schemas.Configuration
                                        ] = None,
                                        **kwargs: typing.Union[
                                            schemas.AnyTypeSchema,
                                            dict,
                                            frozendict.frozendict,
                                            str,
                                            date,
                                            datetime,
                                            uuid.UUID,
                                            int,
                                            float,
                                            decimal.Decimal,
                                            None,
                                            list,
                                            tuple,
                                            bytes,
                                        ],
                                    ) -> "provider":
                                        return super().__new__(
                                            cls,
                                            *_args,
                                            name=name,
                                            invocation=invocation,
                                            modelParams=modelParams,
                                            inputs=inputs,
                                            outputs=outputs,
                                            _configuration=_configuration,
                                            **kwargs,
                                        )

                                elapsedTime = schemas.IntSchema
                                startTime = schemas.DateTimeSchema
                                endTime = schemas.DateTimeSchema
                                __annotations__ = {
                                    "provider": provider,
                                    "elapsedTime": elapsedTime,
                                    "startTime": startTime,
                                    "endTime": endTime,
                                }

                        @typing.overload
                        def __getitem__(
                            self, name: typing_extensions.Literal["provider"]
                        ) -> MetaOapg.properties.provider:
                            ...

                        @typing.overload
                        def __getitem__(
                            self, name: typing_extensions.Literal["elapsedTime"]
                        ) -> MetaOapg.properties.elapsedTime:
                            ...

                        @typing.overload
                        def __getitem__(
                            self, name: typing_extensions.Literal["startTime"]
                        ) -> MetaOapg.properties.startTime:
                            ...

                        @typing.overload
                        def __getitem__(
                            self, name: typing_extensions.Literal["endTime"]
                        ) -> MetaOapg.properties.endTime:
                            ...

                        @typing.overload
                        def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
                            ...

                        def __getitem__(
                            self,
                            name: typing.Union[
                                typing_extensions.Literal[
                                    "provider",
                                    "elapsedTime",
                                    "startTime",
                                    "endTime",
                                ],
                                str,
                            ],
                        ):
                            # dict_instance[name] accessor
                            return super().__getitem__(name)

                        @typing.overload
                        def get_item_oapg(
                            self, name: typing_extensions.Literal["provider"]
                        ) -> typing.Union[MetaOapg.properties.provider, schemas.Unset]:
                            ...

                        @typing.overload
                        def get_item_oapg(
                            self, name: typing_extensions.Literal["elapsedTime"]
                        ) -> typing.Union[
                            MetaOapg.properties.elapsedTime, schemas.Unset
                        ]:
                            ...

                        @typing.overload
                        def get_item_oapg(
                            self, name: typing_extensions.Literal["startTime"]
                        ) -> typing.Union[MetaOapg.properties.startTime, schemas.Unset]:
                            ...

                        @typing.overload
                        def get_item_oapg(
                            self, name: typing_extensions.Literal["endTime"]
                        ) -> typing.Union[MetaOapg.properties.endTime, schemas.Unset]:
                            ...

                        @typing.overload
                        def get_item_oapg(
                            self, name: str
                        ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
                            ...

                        def get_item_oapg(
                            self,
                            name: typing.Union[
                                typing_extensions.Literal[
                                    "provider",
                                    "elapsedTime",
                                    "startTime",
                                    "endTime",
                                ],
                                str,
                            ],
                        ):
                            return super().get_item_oapg(name)

                        def __new__(
                            cls,
                            *_args: typing.Union[
                                dict,
                                frozendict.frozendict,
                            ],
                            provider: typing.Union[
                                MetaOapg.properties.provider,
                                dict,
                                frozendict.frozendict,
                                schemas.Unset,
                            ] = schemas.unset,
                            elapsedTime: typing.Union[
                                MetaOapg.properties.elapsedTime,
                                decimal.Decimal,
                                int,
                                schemas.Unset,
                            ] = schemas.unset,
                            startTime: typing.Union[
                                MetaOapg.properties.startTime,
                                str,
                                datetime,
                                schemas.Unset,
                            ] = schemas.unset,
                            endTime: typing.Union[
                                MetaOapg.properties.endTime,
                                str,
                                datetime,
                                schemas.Unset,
                            ] = schemas.unset,
                            _configuration: typing.Optional[
                                schemas.Configuration
                            ] = None,
                            **kwargs: typing.Union[
                                schemas.AnyTypeSchema,
                                dict,
                                frozendict.frozendict,
                                str,
                                date,
                                datetime,
                                uuid.UUID,
                                int,
                                float,
                                decimal.Decimal,
                                None,
                                list,
                                tuple,
                                bytes,
                            ],
                        ) -> "items":
                            return super().__new__(
                                cls,
                                *_args,
                                provider=provider,
                                elapsedTime=elapsedTime,
                                startTime=startTime,
                                endTime=endTime,
                                _configuration=_configuration,
                                **kwargs,
                            )

                def __new__(
                    cls,
                    _arg: typing.Union[
                        typing.Tuple[
                            typing.Union[
                                MetaOapg.items,
                                dict,
                                frozendict.frozendict,
                            ]
                        ],
                        typing.List[
                            typing.Union[
                                MetaOapg.items,
                                dict,
                                frozendict.frozendict,
                            ]
                        ],
                    ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> "stepRuns":
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )

                def __getitem__(self, i: int) -> MetaOapg.items:
                    return super().__getitem__(i)

            __annotations__ = {
                "id": id,
                "name": name,
                "stepRuns": stepRuns,
            }

    stepRuns: MetaOapg.properties.stepRuns
    name: MetaOapg.properties.name
    id: MetaOapg.properties.id

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["id"]
    ) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["name"]
    ) -> MetaOapg.properties.name:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["stepRuns"]
    ) -> MetaOapg.properties.stepRuns:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(
        self,
        name: typing.Union[
            typing_extensions.Literal[
                "id",
                "name",
                "stepRuns",
            ],
            str,
        ],
    ):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["id"]
    ) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["name"]
    ) -> MetaOapg.properties.name:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["stepRuns"]
    ) -> MetaOapg.properties.stepRuns:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(
        self,
        name: typing.Union[
            typing_extensions.Literal[
                "id",
                "name",
                "stepRuns",
            ],
            str,
        ],
    ):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        stepRuns: typing.Union[
            MetaOapg.properties.stepRuns,
            list,
            tuple,
        ],
        name: typing.Union[
            MetaOapg.properties.name,
            str,
        ],
        id: typing.Union[
            MetaOapg.properties.id,
            str,
            uuid.UUID,
        ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[
            schemas.AnyTypeSchema,
            dict,
            frozendict.frozendict,
            str,
            date,
            datetime,
            uuid.UUID,
            int,
            float,
            decimal.Decimal,
            None,
            list,
            tuple,
            bytes,
        ],
    ) -> "PipelineRunRequest":
        return super().__new__(
            cls,
            *_args,
            stepRuns=stepRuns,
            name=name,
            id=id,
            _configuration=_configuration,
            **kwargs,
        )



from gentrace.providers.getters import configure_openai, configure_pinecone
from gentrace.providers.pipeline import Pipeline
from gentrace.providers.pipeline_run import PipelineRun
from gentrace.providers.step_run import StepRun



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.4.8
    Generated by: https://openapi-generator.tech
"""

import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401
from datetime import date, datetime  # noqa: F401

import frozendict  # noqa: F401
import typing_extensions  # noqa: F401

from gentrace import schemas  # noqa: F401

class PipelineRunResponse(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        class properties:
            pipelineRunId = schemas.StrSchema
            __annotations__ = {
                "pipelineRunId": pipelineRunId,
            }
    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["pipelineRunId"]
    ) -> MetaOapg.properties.pipelineRunId: ...
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    def __getitem__(
        self, name: typing.Union[typing_extensions.Literal["pipelineRunId",], str]
    ):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["pipelineRunId"]
    ) -> typing.Union[MetaOapg.properties.pipelineRunId, schemas.Unset]: ...
    @typing.overload
    def get_item_oapg(
        self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    def get_item_oapg(
        self, name: typing.Union[typing_extensions.Literal["pipelineRunId",], str]
    ):
        return super().get_item_oapg(name)
    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        pipelineRunId: typing.Union[
            MetaOapg.properties.pipelineRunId, str, schemas.Unset
        ] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[
            schemas.AnyTypeSchema,
            dict,
            frozendict.frozendict,
            str,
            date,
            datetime,
            uuid.UUID,
            int,
            float,
            decimal.Decimal,
            None,
            list,
            tuple,
            bytes,
        ],
    ) -> "PipelineRunResponse":
        return super().__new__(
            cls,
            *_args,
            pipelineRunId=pipelineRunId,
            _configuration=_configuration,
            **kwargs,
        )



import inspect
from typing import Any, Optional


class Pipeline:
    def __init__(
        self,
        id: str,
        api_key: str,
        host: Optional[str] = None,
        openai_config: Any = None,
        pinecone_config: Optional[dict] = None,
    ):
        self.id = id
        self.config = {"api_key": api_key, "host": host}

        if openai_config:
            try:
                import openai
            except ImportError:
                raise ValueError(
                    "Could not import OpenAI python package. "
                    "Please install it with `pip install openai`."
                )

            for key in openai_config:
                if key not in openai.__all__:
                    raise ValueError(
                        f"Invalid key ({key}) in supplied OpenAI configuration."
                    )

            self.openai_config = openai_config
        else:
            self.openai_config = None

        if pinecone_config:
            try:
                import pinecone
            except ImportError:
                raise ValueError(
                    "Could not import Pinecone python package. "
                    "Please install it with `pip install pinecone-client`."
                )

            pinecone_init_args = inspect.signature(pinecone.init).parameters.keys()

            for key in pinecone_config:
                if key not in pinecone_init_args:
                    raise ValueError(
                        f"Invalid key ({key}) in supplied Pinecone configuration."
                    )
            self.pinecone_config = pinecone_config
        else:
            self.pinecone_config = None

        self.pipeline_handlers = {}

    def setup(self):
        if self.pinecone_config:
            try:
                from gentrace.providers.vectorstores.pinecone import (
                    PineconePipelineHandler,
                )

                pineconeHandler = PineconePipelineHandler(pipeline=self)
                pineconeHandler.init(api_key=self.pinecone_config["api_key"])
                self.pipeline_handlers["pinecone"] = pineconeHandler
            except ImportError:
                raise ImportError(
                    "Please install Pinecone as a dependency with, e.g. `pip install pinecone-client`"
                )

        if self.openai_config:
            try:
                from gentrace.providers.llms.openai import OpenAIPipelineHandler

                OpenAIPipelineHandler.setup(self.openai_config)
                openai_handler = OpenAIPipelineHandler(
                    self.config, self.openai_config, pipeline=self
                )
                self.pipeline_handlers["openai"] = openai_handler
            except ImportError:
                raise ImportError(
                    "Please install OpenAI as a dependency with, e.g. `pip install openai`"
                )

    def start(self):
        from gentrace.providers.pipeline_run import PipelineRun

        return PipelineRun(pipeline=self)



# coding: utf-8

# flake8: noqa

""" Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.4.8
    Generated by: https://openapi-generator.tech
"""

__version__ = "1.0.0"

# import ApiClient
from gentrace.api_client import ApiClient

# import Configuration
from gentrace.configuration import Configuration

# import exceptions
from gentrace.exceptions import OpenApiException
from gentrace.exceptions import ApiAttributeError
from gentrace.exceptions import ApiTypeError
from gentrace.exceptions import ApiValueError
from gentrace.exceptions import ApiKeyError
from gentrace.exceptions import ApiException

from gentrace.providers.pipeline import Pipeline
from gentrace.providers.pipeline_run import PipelineRun, flush
from gentrace.providers.step_run import StepRun
from gentrace.providers.utils import to_date_string

from gentrace.providers.getters import *

api_key = ""
host = ""



import asyncio
import concurrent
import copy
import inspect
import threading
import uuid
from typing import Dict, List, Optional, cast

from gentrace.api_client import ApiClient
from gentrace.apis.tags.ingestion_api import IngestionApi
from gentrace.configuration import Configuration
from gentrace.providers.pipeline import Pipeline
from gentrace.providers.step_run import StepRun
from gentrace.providers.utils import pipeline_run_post_background

_pipeline_run_loop = None
_pipeline_tasks = []


# https://stackoverflow.com/a/63110035/1057411
def fire_and_forget(coro):
    global _pipeline_run_loop, _pipeline_tasks
    if _pipeline_run_loop is None:
        _pipeline_run_loop = asyncio.new_event_loop()
        threading.Thread(target=_pipeline_run_loop.run_forever, daemon=True).start()
    if inspect.iscoroutine(coro):
        task = asyncio.run_coroutine_threadsafe(coro, _pipeline_run_loop)
        _pipeline_tasks.append(task)


def flush():
    global _pipeline_tasks
    if _pipeline_tasks:
        # Wait for all tasks to complete
        concurrent.futures.wait(_pipeline_tasks)
        _pipeline_tasks.clear()


class PipelineRun:
    def __init__(self, pipeline, id: Optional[str] = None):
        self.pipeline: Pipeline = pipeline
        self.pipeline_run_id: str = id or str(uuid.uuid4())
        self.step_runs: List[StepRun] = []

    def get_pipeline(self):
        return self.pipeline

    def get_openai(self):
        if "openai" in self.pipeline.pipeline_handlers:
            handler = self.pipeline.pipeline_handlers.get("openai")
            cloned_handler = copy.deepcopy(handler)

            from .llms.openai import annotate_pipeline_handler

            annotated_handler = annotate_pipeline_handler(
                cloned_handler, self.pipeline.openai_config, self
            )

            import openai

            # TODO: Could not find an easy way to create a union type with openai and
            # OpenAIPipelineHandler, so we just use openai.
            typed_cloned_handler = cast(openai, annotated_handler)

            return typed_cloned_handler
        else:
            raise ValueError(
                "Did not find OpenAI handler. Did you call setup() on the pipeline?"
            )

    def get_pinecone(self):
        if "pinecone" in self.pipeline.pipeline_handlers:
            handler = self.pipeline.pipeline_handlers.get("pinecone")
            cloned_handler = copy.deepcopy(handler)
            cloned_handler.set_pipeline_run(self)
            import pinecone

            return cast(pinecone, cloned_handler)
        else:
            raise ValueError(
                "Did not find Pinecone handler. Did you call setup() on the pipeline?"
            )

    def add_step_run(self, step_run: StepRun):
        self.step_runs.append(step_run)

    async def asubmit(self) -> Dict:
        configuration = Configuration(host=self.pipeline.config.get("host"))
        configuration.access_token = self.pipeline.config.get("api_key")
        api_client = ApiClient(configuration=configuration)
        ingestion_api = IngestionApi(api_client=api_client)

        step_runs_data = [
            {
                "provider": {
                    "name": step_run.provider,
                    "invocation": step_run.invocation,
                    "modelParams": step_run.model_params,
                    "inputs": step_run.inputs,
                    "outputs": step_run.outputs,
                },
                "elapsedTime": step_run.elapsed_time,
                "startTime": step_run.start_time,
                "endTime": step_run.end_time,
            }
            for step_run in self.step_runs
        ]

        pipeline_run_id = str(uuid.uuid4())

        try:
            pipeline_post_response = await pipeline_run_post_background(
                ingestion_api,
                {
                    "id": pipeline_run_id,
                    "name": self.pipeline.id,
                    "stepRuns": step_runs_data,
                },
            )
            return {
                "pipelineRunId": pipeline_post_response.body.get_item_oapg(
                    "pipelineRunId"
                )
            }
        except Exception as e:
            print(f"Error submitting to Gentrace: {e}")
            return {"pipelineRunId": None}

    def submit(self, wait_for_server=False) -> Dict:
        configuration = Configuration(host=self.pipeline.config.get("host"))
        configuration.access_token = self.pipeline.config.get("api_key")

        api_client = ApiClient(configuration=configuration)
        ingestion_api = IngestionApi(api_client=api_client)

        step_runs_data = [
            {
                "provider": {
                    "name": step_run.provider,
                    "invocation": step_run.invocation,
                    "modelParams": step_run.model_params,
                    "inputs": step_run.inputs,
                    "outputs": step_run.outputs,
                },
                "elapsedTime": step_run.elapsed_time,
                "startTime": step_run.start_time,
                "endTime": step_run.end_time,
            }
            for step_run in self.step_runs
        ]

        if len(step_runs_data) == 0:
            return {"pipelineRunId": None}

        if not wait_for_server:
            fire_and_forget(
                pipeline_run_post_background(
                    ingestion_api,
                    {
                        "id": self.pipeline_run_id,
                        "name": self.pipeline.id,
                        "stepRuns": step_runs_data,
                    },
                )
            )

            return {"pipelineRunId": self.pipeline_run_id}

        if wait_for_server:
            try:
                pipeline_post_response = ingestion_api.pipeline_run_post(
                    {
                        "id": self.pipeline_run_id,
                        "name": self.pipeline.id,
                        "stepRuns": step_runs_data,
                    }
                )
                return {
                    "pipelineRunId": pipeline_post_response.body.get_item_oapg(
                        "pipelineRunId"
                    )
                }

            except Exception as e:
                print(f"Error submitting to Gentrace: {e}")
                return {"pipelineRunId": None}



import typing_extensions

from gentrace.apis.paths.pipeline_run import PipelineRun
from gentrace.paths import PathValues

PathToApi = typing_extensions.TypedDict(
    "PathToApi",
    {
        PathValues.PIPELINERUN: PipelineRun,
    },
)

path_to_api = PathToApi(
    {
        PathValues.PIPELINERUN: PipelineRun,
    }
)



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from gentrace.apis.tag_to_api import tag_to_api

import enum


class TagValues(str, enum.Enum):
    INGESTION = "Ingestion"
    FEEDBACK = "Feedback"



import os

import openai

from gentrace.configuration import Configuration as GentraceConfiguration

openai.api_key = os.getenv("OPENAI_KEY")


def configure_openai():
    from gentrace import api_key, host

    from .llms.openai import annotate_openai_module

    if not api_key:
        raise ValueError("Gentrace API key not set")

    gentrace_config = GentraceConfiguration(host=host)
    gentrace_config.access_token = api_key

    annotate_openai_module(gentrace_config=gentrace_config)


def configure_pinecone():
    from .vectorstores.pinecone import annotate_pinecone_module

    annotate_pinecone_module()


__all__ = ["configure_openai", "configure_pinecone"]



# coding: utf-8

# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from gentrace.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from gentrace.model.feedback_request import FeedbackRequest
from gentrace.model.feedback_response import FeedbackResponse
from gentrace.model.pipeline_run_request import PipelineRunRequest
from gentrace.model.pipeline_run_response import PipelineRunResponse



import asyncio
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime

from gentrace.apis.tags.ingestion_api import IngestionApi
from gentrace.models import PipelineRunRequest

__all__ = ["to_date_string"]


def to_date_string(time_value):
    return (
        datetime.fromtimestamp(time_value).strftime("%Y-%m-%dT%H:%M:%S.%fZ")[:-4] + "Z"
    )


async def pipeline_run_post_background(
    api_instance: IngestionApi, pipeline_run_data: PipelineRunRequest
):
    with ThreadPoolExecutor() as executor:
        result = await asyncio.get_event_loop().run_in_executor(
            executor, api_instance.pipeline_run_post, pipeline_run_data
        )
    return result



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.4.8
    Generated by: https://openapi-generator.tech
"""

from gentrace.paths.pipeline_run.post import PipelineRunPost


class IngestionApi(
    PipelineRunPost,
):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    pass
